﻿#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// Define constants
#define PI 3.14159265358979323846

// Define struct for bubble data
typedef struct {
    double zPos, radius;
} BubbleData;

// Function prototypes
void equalSlices(int slices); void cheeseDivision(BubbleData bubbles[], double adjustedVolume, int numBubbles, int slices); double computeSliceVolume(BubbleData bubbles[], int numBubbles, double minHeight, double maxHeight);
double sphereVolume(double radius); double sphereCapVolume(double radius, double capHeight);



double findBubbleVolume(BubbleData bubble, double minHeight, double maxHeight);

int main() {
    int numBubbles, slices;
    scanf("%d %d", &numBubbles, &slices);
    BubbleData bubbles[numBubbles + 1];

    // Read bubble data and convert units
    double bufferX, bufferY;
    for (int i = 0; i < numBubbles; i++) {
        scanf("%lf %lf %lf %lf", &(bubbles[i].radius), &bufferX, &bufferY, &(bubbles[i].zPos));
        bubbles[i].radius /= 1000; bubbles[i].zPos /= 1000;
    }

    // Handle special cases
    if (numBubbles == 0) {
        equalSlices(slices); return 0;
    } else if (slices == 1) {
        printf("100\n"); return 0;
    }

    // Calculate total bubble volume
    double totalBubbleVolume = 0;
    for (int i = 0; i < numBubbles; i++) {
        totalBubbleVolume += sphereVolume(bubbles[i].radius);
    }
    double adjustedVolume = (1000000 - totalBubbleVolume) / slices;

    // Divide cheese
    cheeseDivision(bubbles, adjustedVolume, numBubbles, slices); return 0;
}

// Function to divide cheese with bubbles
void cheeseDivision(BubbleData bubbles[], double adjustedVolume, int numBubbles, int slices) {
    double prevLowerBound = 0, lastSliceHeight = 100;

    for (int i = 0; i < slices - 1; i++) {
        double lowerBound = prevLowerBound;
        double upperBound = 100;
        double center;
        double currentVolume;
        int iterations = 0;

        do {
            center = (lowerBound + upperBound) / 2;
            currentVolume = computeSliceVolume(bubbles, numBubbles, prevLowerBound, center);

            if (currentVolume < adjustedVolume - 1e-8) {
                upperBound = center;
            }
            else {
                lowerBound = center;
            }
            iterations++;
        } while (fabs(currentVolume - adjustedVolume) > 1e-8 && iterations < 1000);

        printf("%.9lf\n", center - prevLowerBound);
        lastSliceHeight -= center - prevLowerBound;
        prevLowerBound = center;
    }

    printf("%.9lf\n", lastSliceHeight);
}

// Function to compute volume of cheese slice
double computeSliceVolume(BubbleData bubbles[], int numBubbles, double minHeight, double maxHeight) {
    double totalBubbleVolume = 0;
    for (int i = 0; i < numBubbles; i++) {
        totalBubbleVolume += findBubbleVolume(bubbles[i], minHeight, maxHeight);
    }
    double cheeseVolume = 100 * 100 * (maxHeight - minHeight);

    return cheeseVolume - totalBubbleVolume;
}

// Function to find volume of bubble inside slice
double findBubbleVolume(BubbleData bubble, double minHeight, double maxHeight) {
    
    if (bubble.zPos - bubble.radius < minHeight && maxHeight < bubble.zPos + bubble.radius) {
        
        double vol = sphereVolume(bubble.radius);
        vol -= sphereCapVolume(bubble.radius, fabs(minHeight - bubble.zPos + bubble.radius)); vol -= sphereCapVolume(bubble.radius, fabs(maxHeight - bubble.zPos - bubble.radius));
        return vol;
    
    
    } else if (bubble.zPos - bubble.radius < maxHeight && maxHeight < bubble.zPos + bubble.radius) {
        return sphereCapVolume(bubble.radius, fabs(maxHeight - bubble.zPos + bubble.radius));
    
    
    } else if (bubble.zPos - bubble.radius < minHeight && minHeight < bubble.zPos + bubble.radius) {
        return sphereCapVolume(bubble.radius, fabs(minHeight - bubble.zPos - bubble.radius));
    
    
    } else if (minHeight <= bubble.zPos - bubble.radius && bubble.zPos + bubble.radius <= maxHeight) {
        return sphereVolume(bubble.radius);
    }
    return 0;
}

// Function to print equal slices
void equalSlices(int slices) {
    for (int i = 0; i < slices; i++) { printf("%.9lf\n", (double)100 / slices);
    }
}

// Function to calculate volume of a sphere
double sphereVolume(double radius) {
    return (4 * pow(radius, 3) * PI) / 3;
}

// Function to calculate volume of a spherical cap
double sphereCapVolume(double radius, double capHeight) {
    
    
    
    
    return (PI * pow(capHeight, 2)) * (radius - (capHeight / 3));
}
