﻿#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>

// Define a node for the binary search tree
struct Node {
    int data;
    struct Node* left;
    struct Node* right;
};

// Function to create a new node
struct Node* newNode(int data) {
    struct Node* temp = (struct Node*)malloc(sizeof(struct Node));
    temp->data = data;
    temp->left = temp->right = NULL;
    return temp;
}

// Function to insert a value into the binary search tree
struct Node* insert(struct Node* root, int value) {
    if (root == NULL) {
        return newNode(value);
    }

    if (value < root->data) {
        root->left = insert(root->left, value);
    } else {
        root->right = insert(root->right, value);
    }

    return root;
}

// Function to perform preorder traversal (optional for serialization)
void preorderTraversal(struct Node* root) {
    if (root) {
        printf("%d ", root->data);
        preorderTraversal(root->left);
        preorderTraversal(root->right);
    } else {
        printf("# "); // Print special character for empty subtree
    }
}

// **Optional function for comparing tree equality (advanced)**
bool isIdentical(struct Node* root1, struct Node* root2) {
    if (root1 == root2) {
        return true;
    }
    if (root1 == NULL || root2 == NULL) {
        return false;
    }
    return root1->data == root2->data &&
           isIdentical(root1->left, root2->left) &&
           isIdentical(root1->right, root2->right);
}

// Function to count unique tree shapes (using set for efficiency)
void serializeTree(struct Node root, char buffer) {
    if (!root) {
        strcat(buffer, "#");
        return;
    }
    char num[12];
    sprintf(num, "%d", root->data);
    strcat(buffer, num);
    serializeTree(root->left, buffer);
    serializeTree(root->right, buffer);
}

int countUniqueShapes(int n, int k, int prototypes[][k]) {
    char shapes[1000][1000] = { 0 }; 
    int count = 0;

    for (int i = 0; i < n; i++) {
        struct Node* root = NULL;
        for (int j = 0; j < k; j++) {
            root = insert(root, prototypes[i][j]);
        }

        char fingerprint[1000] = "";
        serializeTree(root, fingerprint);

        int isUnique = 1;
        for (int j = 0; j < count; j++) {
            if (strcmp(shapes[j], fingerprint) == 0) {
                isUnique = 0;
                break;
            }
        }

        if (isUnique) {
            strcpy(shapes[count], fingerprint);
            count++;
        }
    }

    return count;
}

int main() {
    int n, k;
    scanf("%d %d", &n, &k);

    int prototypes[n][k];
    for (int i = 0; i < n; i++) {
        for (int j = 0; j < k; j++) {
            scanf("%d", &prototypes[i][j]);
        }
    }

    int uniqueShapes = countUniqueShapes(n, k, prototypes);
    printf("%d\n", uniqueShapes);

    return 0;
}
